<?php

namespace Simple_Giveaways;

/**
 * Class SG_Action_Visit
 */
class SG_Action_Premium_Dummy extends SG_Extra_Action {

	/**
	 * SG_Action_Visit constructor.
	 */
	public function __construct() {
		$this->slug        = '';
		$this->category    = 'premium';
		$this->title       = __( 'Premium Option', 'giveasap' );
		$this->description = __( 'Rewards the user for a premium action.', 'giveasap' );
		$this->icon        = 'fal fa-link';
	}

	/**
	 * Return the description.
	 *
	 * @return string
	 */
	public function get_description() {
		$url = '<a href="' . giv_fs()->get_upgrade_url() . '"
                               class="button button-primary">' . __( 'Upgrade to PRO', 'giveasap' ) . '</a>';
		return $this->description . $url;
	}

	/**
	 * Get the admin form.
	 *
	 * @param int $index
	 *
	 * @return string
	 */
	public function get_admin_form( $index = 0 ) {
		return '';
	}
}

/**
 * Class SG_Action_Visit
 */
class SG_Action_Business_Dummy extends SG_Action_Premium_Dummy {

	/**
	 * Return the description.
	 *
	 * @return string
	 */
	public function get_description() {
		$url = '<a href="' . giv_fs()->get_upgrade_url() . '"
                               class="button button-primary">' . __( 'Upgrade to Business', 'giveasap' ) . '</a>';
		return $this->description . $url;
	}

}

class SG_Action_Premium_Dummy_Facebook_Visit extends SG_Action_Premium_Dummy {
	/**
	 * constructor.
	 */
	public function __construct() {

		parent::__construct();

		$this->title       = __( 'Facebook Visit - Premium', 'giveasap' );
		$this->description = __( 'Rewards the user for visiting a facebook page/profile.', 'giveasap' );
		$this->icon        = 'fab fa-facebook';

	}
}

class SG_Action_Premium_Dummy_Twitter_Follow extends SG_Action_Premium_Dummy {
	/**
	 * constructor.
	 */
	public function __construct() {

		parent::__construct();

		$this->title       = __( 'Twitter Follow - Premium', 'giveasap' );
		$this->description = __( 'Rewards the user for following a twitter profile. Needs Twitter Integration activated.', 'giveasap' );
		$this->icon        = 'fab fa-twitter';
		$this->category    = 'twitter';

	}

	/**
	 * Return the description.
	 *
	 * @return string
	 */
	public function get_description() {
		$url = '<a href="' . admin_url( 'edit.php?post_type=giveasap&page=giveasap_integrations' ) . '"
                               class="button button-primary">' . __( 'Activate Twitter Integration', 'giveasap' ) . '</a>';
		return $this->description . $url;
	}
}

class SG_Action_Premium_Dummy_Twitter_Tweet extends SG_Action_Premium_Dummy_Twitter_Follow {
	/**
	 * constructor.
	 */
	public function __construct() {

		parent::__construct();

		$this->title       = __( 'Twitter Tweet - Premium', 'giveasap' );
		$this->description = __( 'Rewards the user for tweeting a status. Needs Twitter Integration activated.', 'giveasap' );
		$this->icon        = 'fab fa-twitter';
		$this->category    = 'twitter';

	}
}

class SG_Action_Premium_Dummy_Twitter_Visit extends SG_Action_Premium_Dummy {
	/**
	 * constructor.
	 */
	public function __construct() {

		parent::__construct();

		$this->title       = __( 'Twitter Visit - Premium', 'giveasap' );
		$this->description = __( 'Rewards the user for visiting a twitter profile.', 'giveasap' );
		$this->icon        = 'fab fa-twitter';
		$this->category    = 'twitter';

	}
}

class SG_Action_Premium_Dummy_Instagram_Visit extends SG_Action_Premium_Dummy {
	/**
	 * constructor.
	 */
	public function __construct() {

		parent::__construct();

		$this->title       = __( 'Instagram Visit - Premium', 'giveasap' );
		$this->description = __( 'Rewards the user for visiting an instagram profile.', 'giveasap' );
		$this->icon        = 'fab fa-instagram';

	}
}

class SG_Action_Premium_Dummy_Question extends SG_Action_Premium_Dummy {
	/**
	 * constructor.
	 */
	public function __construct() {

		parent::__construct();

		$this->slug        = 'question';
		$this->category    = 'general';
		$this->title       = __( 'Answer a Question', 'giveasap' );
		$this->description = __( 'Rewards the user for giving an answer.', 'giveasap' );
		$this->icon        = 'fal fa-question-circle';

	}
}

class SG_Action_Premium_Dummy_Secret_Code extends SG_Action_Premium_Dummy {
	/**
	 * constructor.
	 */
	public function __construct() {

		parent::__construct();

		$this->slug        = 'secret_code';
		$this->category    = 'general';
		$this->title       = __( 'Secret Code', 'giveasap' );
		$this->description = __( 'Rewards the user for entering the correct code.', 'giveasap' );
		$this->icon        = 'fal fa-key';

	}
}

class SG_Action_Premium_Dummy_Invent_Own extends SG_Action_Premium_Dummy {
	/**
	 * constructor.
	 */
	public function __construct() {

		parent::__construct();

		$this->slug        = 'invent_own';
		$this->category    = 'general';
		$this->title       = __( 'Invent Own', 'giveasap' );
		$this->description = __( 'Invent your own action', 'giveasap' );
		$this->icon        = 'fal fa-brain';

	}
}

class SG_Action_Premium_Dummy_Watch_Youtube extends SG_Action_Premium_Dummy {
	/**
	 * constructor.
	 */
	public function __construct() {

		parent::__construct();

		$this->slug        = 'youtube_watch';
		$this->category    = 'youtube';
		$this->title       = __( 'Watch YouTube', 'giveasap' );
		$this->description = __( 'Rewards a user for watching a YouTube video', 'giveasap' );
		$this->icon        = 'fab fa-youtube';

	}
}

/**
 * Class SG_Action_Visit
 */
class SG_Action_Business_Dummy_WooCommerce_Purchased extends SG_Action_Premium_Dummy {

	/**
	 * constructor.
	 */
	public function __construct() {

		parent::__construct();

		$this->slug        = 'woocommerce_purchased';
		$this->category    = 'woocommerce';
		$this->title       = __( 'Purchased Product', 'giveasap' );
		$this->description = __( 'Rewards the user for purchasing a product.', 'giveasap' );
		$this->icon        = 'fal fa-store';

	}


	/**
	 * Return the description.
	 *
	 * @return string
	 */
	public function get_description() {
		$url = '<a href="' . admin_url( 'edit.php?post_type=giveasap&page=giveasap_integrations' ) . '"
                               class="button button-primary">' . __( 'Activate WooCommerce Integration', 'giveasap' ) . '</a>';
		return $this->description . $url;
	}
}

/**
 * Class SG_Action_Visit
 */
class SG_Action_Business_Dummy_EDD_Purchased extends SG_Action_Premium_Dummy {

	/**
	 * constructor.
	 */
	public function __construct() {

		parent::__construct();

		$this->slug        = 'edd_purchased';
		$this->category    = 'edd';
		$this->title       = __( 'Purchased Product', 'giveasap' );
		$this->description = __( 'Rewards the user for purchasing a download.', 'giveasap' );
		$this->icon        = 'fal fa-store';

	}


	/**
	 * Return the description.
	 *
	 * @return string
	 */
	public function get_description() {
		$url = '<a href="' . admin_url( 'edit.php?post_type=giveasap&page=giveasap_integrations' ) . '"
                               class="button button-primary">' . __( 'Activate Easy Digital Downloads Integration', 'giveasap' ) . '</a>';
		return $this->description . $url;
	}
}

/**
 * Class SG_Action_Visit
 */
class SG_Action_Premium_MailChimp_Signup extends SG_Action_Premium_Dummy {

	/**
	 * constructor.
	 */
	public function __construct() {

		parent::__construct();

		$this->slug        = 'mailchimp_signup';
		$this->category    = 'newsletter';
		$this->title       = __( 'Subscribe to MailChimp list', 'giveasap' );
		$this->description = __( 'Rewards the user for subscribing to a MailChimp list.', 'giveasap' );
		$this->icon        = 'fab fa-mailchimp';

	}


	/**
	 * Return the description.
	 *
	 * @return string
	 */
	public function get_description() {
		$url = '<a href="' . admin_url( 'edit.php?post_type=giveasap&page=giveasap_integrations' ) . '"
                               class="button button-primary">' . __( 'Activate MailChimp Integration', 'giveasap' ) . '</a>';
		return $this->description . $url;
	}
}

/**
 * Class SG_Action_Visit
 */
class SG_Action_Business_Dummy_Give_Purchased extends SG_Action_Premium_Dummy {

	/**
	 * constructor.
	 */
	public function __construct() {

		parent::__construct();

		$this->slug        = 'give_donated';
		$this->category    = 'give';
		$this->title       = __( 'Purchased a Donation', 'giveasap' );
		$this->description = __( 'Rewards the user for purchasing a donation.', 'giveasap' );
		$this->icon        = 'fal fa-store';

	}


	/**
	 * Return the description.
	 *
	 * @return string
	 */
	public function get_description() {
		$url = '<a href="' . admin_url( 'edit.php?post_type=giveasap&page=giveasap_integrations' ) . '"
                               class="button button-primary">' . __( 'Activate GiveWP Integration', 'giveasap' ) . '</a>';
		return $this->description . $url;
	}
}
